@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 A collection of utilities for canonicalizing and inspecting graphs.
 
@@ -73,7 +72,8 @@ Only in second::
     _:cb558f30e21ddfc05ca53108348338ade8
         <http://example.org/ns#label> "B" .
 """
-from __future__ import absolute_import, division, print_function
+
+from __future__ import annotations
 
 # TODO:
 # - Doesn't handle quads.
@@ -90,19 +90,10 @@ __all__ = [
 ]
 
 from collections import defaultdict
+from collections.abc import Callable, Iterator
 from datetime import datetime
 from hashlib import sha256
-from typing import (
-    TYPE_CHECKING,
-    Callable,
-    Dict,
-    Iterator,
-    List,
-    Optional,
-    Set,
-    Tuple,
-    Union,
-)
+from typing import TYPE_CHECKING, Optional, Union
 
 from rdflib.graph import ConjunctiveGraph, Graph, ReadOnlyGraphAggregate, _TripleType
 from rdflib.term import BNode, IdentifiedNode, Node, URIRef
@@ -118,7 +109,7 @@ def _total_seconds(td):
     return result
 
 
-class _runtime(object):
+class _runtime:  # noqa: N801
     def __init__(self, label):
         self.label = label
 
@@ -137,7 +128,7 @@ class _runtime(object):
         return wrapped_f
 
 
-class _call_count(object):
+class _call_count:  # noqa: N801
     def __init__(self, label):
         self.label = label
 
@@ -202,16 +193,16 @@ class IsomorphicGraph(ConjunctiveGraph):
 
 
 HashFunc = Callable[[str], int]
-ColorItem = Tuple[Union[int, str], URIRef, Union[int, str]]
-ColorItemTuple = Tuple[ColorItem, ...]
-HashCache = Optional[Dict[ColorItemTuple, str]]
-Stats = Dict[str, Union[int, str]]
+ColorItem = tuple[Union[int, str], URIRef, Union[int, str]]
+ColorItemTuple = tuple[ColorItem, ...]
+HashCache = Optional[dict[ColorItemTuple, str]]
+Stats = dict[str, Union[int, str]]
 
 
 class Color:
     def __init__(
         self,
-        nodes: List[IdentifiedNode],
+        nodes: list[IdentifiedNode],
         hashfunc: HashFunc,
         color: ColorItemTuple = (),
         hash_cache: HashCache = None,
@@ -231,7 +222,7 @@ class Color:
     def key(self):
         return (len(self.nodes), self.hash_color())
 
-    def hash_color(self, color: Optional[Tuple[ColorItem, ...]] = None) -> str:
+    def hash_color(self, color: tuple[ColorItem, ...] | None = None) -> str:
         if color is None:
             color = self.color
         if color in self._hash_cache:
@@ -252,10 +243,10 @@ class Color:
         self._hash_cache[color] = val
         return val
 
-    def distinguish(self, W: "Color", graph: Graph):
-        colors: Dict[str, Color] = {}
+    def distinguish(self, W: Color, graph: Graph):  # noqa: N803
+        colors: dict[str, Color] = {}
         for n in self.nodes:
-            new_color: Tuple[ColorItem, ...] = list(self.color)  # type: ignore[assignment]
+            new_color: tuple[ColorItem, ...] = list(self.color)  # type: ignore[assignment]
             for node in W.nodes:
                 new_color += [  # type: ignore[operator]
                     (1, p, W.hash_color()) for s, p, o in graph.triples((n, None, node))
@@ -284,7 +275,7 @@ class Color:
 _HashT = Callable[[], "HASH"]
 
 
-class _TripleCanonicalizer(object):
+class _TripleCanonicalizer:
     def __init__(self, graph: Graph, hashfunc: _HashT = sha256):
         self.graph = graph
 
@@ -296,10 +287,10 @@ class _TripleCanonicalizer(object):
         self._hash_cache: HashCache = {}
         self.hashfunc = _hashfunc
 
-    def _discrete(self, coloring: List[Color]) -> bool:
+    def _discrete(self, coloring: list[Color]) -> bool:
         return len([c for c in coloring if not c.discrete()]) == 0
 
-    def _initial_color(self) -> List[Color]:
+    def _initial_color(self) -> list[Color]:
         """Finds an initial color for the graph.
 
         Finds an initial color of the graph by finding all blank nodes and
@@ -307,7 +298,7 @@ class _TripleCanonicalizer(object):
         nodes are not included, as they are a) already colored (by URI or literal)
         and b) do not factor into the color of any blank node.
         """
-        bnodes: Set[BNode] = set()
+        bnodes: set[BNode] = set()
         others = set()
         self._neighbors = defaultdict(set)
         for s, p, o in self.graph:
@@ -343,16 +334,16 @@ class _TripleCanonicalizer(object):
         )
         return c
 
-    def _get_candidates(self, coloring: List[Color]) -> Iterator[Tuple[Node, Color]]:
+    def _get_candidates(self, coloring: list[Color]) -> Iterator[tuple[Node, Color]]:
         for c in [c for c in coloring if not c.discrete()]:
             for node in c.nodes:
                 yield node, c
 
-    def _refine(self, coloring: List[Color], sequence: List[Color]) -> List[Color]:
+    def _refine(self, coloring: list[Color], sequence: list[Color]) -> list[Color]:
         sequence = sorted(sequence, key=lambda x: x.key(), reverse=True)
         coloring = coloring[:]
         while len(sequence) > 0 and not self._discrete(coloring):
-            W = sequence.pop()
+            W = sequence.pop()  # noqa: N806
             for c in coloring[:]:
                 if len(c.nodes) > 1 or isinstance(c.nodes[0], BNode):
                     colors = sorted(
@@ -367,8 +358,8 @@ class _TripleCanonicalizer(object):
                         sequence = sequence[:si] + colors + sequence[si + 1 :]
                     except ValueError:
                         sequence = colors[1:] + sequence
-        combined_colors: List[Color] = []
-        combined_color_map: Dict[str, Color] = dict()
+        combined_colors: list[Color] = []
+        combined_color_map: dict[str, Color] = dict()
         for color in coloring:
             color_hash = color.hash_color()
             # This is a hash collision, and be combined into a single color for individuation.
@@ -380,7 +371,7 @@ class _TripleCanonicalizer(object):
         return combined_colors
 
     @_runtime("to_hash_runtime")
-    def to_hash(self, stats: Optional[Stats] = None):
+    def to_hash(self, stats: Stats | None = None):
         result = 0
         for triple in self.canonical_triples(stats=stats):
             result += self.hashfunc(" ".join([x.n3() for x in triple]))
@@ -388,7 +379,7 @@ class _TripleCanonicalizer(object):
             stats["graph_digest"] = "%x" % result
         return result
 
-    def _experimental_path(self, coloring: List[Color]) -> List[Color]:
+    def _experimental_path(self, coloring: list[Color]) -> list[Color]:
         coloring = [c.copy() for c in coloring]
         while not self._discrete(coloring):
             color = [x for x in coloring if not x.discrete()][0]
@@ -400,9 +391,9 @@ class _TripleCanonicalizer(object):
 
     def _create_generator(
         self,
-        colorings: List[List[Color]],
-        groupings: Optional[Dict[Node, Set[Node]]] = None,
-    ) -> Dict[Node, Set[Node]]:
+        colorings: list[list[Color]],
+        groupings: dict[Node, set[Node]] | None = None,
+    ) -> dict[Node, set[Node]]:
         if not groupings:
             groupings = defaultdict(set)
         for group in zip(*colorings):
@@ -416,20 +407,20 @@ class _TripleCanonicalizer(object):
     @_call_count("individuations")
     def _traces(
         self,
-        coloring: List[Color],
-        stats: Optional[Stats] = None,
-        depth: List[int] = [0],
-    ) -> List[Color]:
+        coloring: list[Color],
+        stats: Stats | None = None,
+        depth: list[int] = [0],
+    ) -> list[Color]:
         if stats is not None and "prunings" not in stats:
             stats["prunings"] = 0
         depth[0] += 1
         candidates = self._get_candidates(coloring)
-        best: List[List[Color]] = []
+        best: list[list[Color]] = []
         best_score = None
         best_experimental_score = None
         last_coloring = None
-        generator: Dict[Node, Set[Node]] = defaultdict(set)
-        visited: Set[Node] = set()
+        generator: dict[Node, set[Node]] = defaultdict(set)
+        visited: set[Node] = set()
         for candidate, color in candidates:
             if candidate in generator:
                 v = generator[candidate] & visited
@@ -437,7 +428,7 @@ class _TripleCanonicalizer(object):
                     visited.add(candidate)
                     continue
             visited.add(candidate)
-            coloring_copy: List[Color] = []
+            coloring_copy: list[Color] = []
             color_copy = None
             for c in coloring:
                 c_copy = c.copy()
@@ -469,7 +460,7 @@ class _TripleCanonicalizer(object):
                 # prune this branch.
                 if stats is not None:
                     stats["prunings"] += 1
-        discrete: List[List[Color]] = [x for x in best if self._discrete(x)]
+        discrete: list[list[Color]] = [x for x in best if self._discrete(x)]
         if len(discrete) == 0:
             best_score = None
             best_depth = None
@@ -484,7 +475,7 @@ class _TripleCanonicalizer(object):
             depth[0] = best_depth  # type: ignore[assignment]
         return discrete[0]
 
-    def canonical_triples(self, stats: Optional[Stats] = None):
+    def canonical_triples(self, stats: Stats | None = None):
         if stats is not None:
             start_coloring = datetime.now()
         coloring = self._initial_color()
@@ -509,7 +500,7 @@ class _TripleCanonicalizer(object):
         if stats is not None:
             stats["color_count"] = len(coloring)
 
-        bnode_labels: Dict[Node, str] = dict(
+        bnode_labels: dict[Node, str] = dict(
             [(c.nodes[0], c.hash_color()) for c in coloring]
         )
         if stats is not None:
@@ -522,8 +513,8 @@ class _TripleCanonicalizer(object):
 
     def _canonicalize_bnodes(
         self,
-        triple: "_TripleType",
-        labels: Dict[Node, str],
+        triple: _TripleType,
+        labels: dict[Node, str],
     ):
         for term in triple:
             if isinstance(term, BNode):
@@ -578,9 +569,7 @@ def isomorphic(graph1: Graph, graph2: Graph) -> bool:
     return gd1 == gd2
 
 
-def to_canonical_graph(
-    g1: Graph, stats: Optional[Stats] = None
-) -> ReadOnlyGraphAggregate:
+def to_canonical_graph(g1: Graph, stats: Stats | None = None) -> ReadOnlyGraphAggregate:
     """Creates a canonical, read-only graph.
 
     Creates a canonical, read-only graph where all bnode id:s are based on
@@ -591,7 +580,7 @@ def to_canonical_graph(
     return ReadOnlyGraphAggregate([graph])
 
 
-def graph_diff(g1: Graph, g2: Graph) -> Tuple[Graph, Graph, Graph]:
+def graph_diff(g1: Graph, g2: Graph) -> tuple[Graph, Graph, Graph]:
     """Returns three sets of triples: "in both", "in first" and "in second"."""
     # bnodes have deterministic values in canonical graphs:
     cg1 = to_canonical_graph(g1)
@@ -619,7 +608,7 @@ def similar(g1: Graph, g2: Graph):
 
 
 def _squashed_graphs_triples(g1: Graph, g2: Graph):
-    for (t1, t2) in zip(sorted(_squash_graph(g1)), sorted(_squash_graph(g2))):
+    for t1, t2 in zip(sorted(_squash_graph(g1)), sorted(_squash_graph(g2))):
         yield t1, t2
 
 
